﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

/*
 ScriptableObjects definem a estrutura de um tipo de asset
 novo.
 SOs permitem que os programadores armazenem informações do
 jogo de uma forma mais simples que um arquivo JSON ou XML
 para dados IMUTÁVEIS (ver comentários em SaveSystem.cs).
 SOs são uma das opções para, por exemplo, guardar layout
 de níveis, árvores de diálogo ou, neste caso, informações
 de inimigos diferentes que usem o mesmo prefab para manter
 consistência.
 Criar um prefab para cada inimigo arrisca gerar problemas no
 futuro. Modificar um e não modificar outro pode levar a
 diferenças inesperadas no comportamento, ou modificar um script
 que pensava estar envolvido apenas em um objeto pode afetar os
 outros também.
 Separando a informação do comportamento, evitamos esses problemas.
 */
[CreateAssetMenu(
	menuName="Meu Joguin/Coisas/Enemy Type")]
public class EnemyInfo : ScriptableObject // Note que não herda de MonoBehaviour
{
	public int hpMax;
	public Color color;
	public int awardedExperience;

	/*
	 Usar /// permite que a sua IDE (Visual Studio, VS Code, etc.) exiba
	 a descrição do método ou atributo quando passar o mouse por cima.
	 Veja como fica: este método é usado em Spawner.cs.
	 Isso não é exclusivo de ScriptableObjects. Funciona também para
	 qualquer classe, mesmo aquelas nem SO nem MonoBehaviour.
	 Usar esse tipo de comentário é um jeito rápido de documentar
	 seu código, e ainda mais rápido para consultá-la.
	 */
	/// <summary>
	/// Altera os componentes do GameObject dado com as informações no
	/// ScriptableObject.
	/// </summary>
	/// <param name="obj">O objeto a ser alterado</param>
	public void Apply(GameObject obj)
	{
		var hp = obj.GetComponent<HP>();
		hp.max = this.hpMax;

		var renderer = obj.GetComponent<Renderer>();
		renderer.material.color = this.color;

		var awardedExp = obj.GetComponent<AwardExpOnDeath>();
		awardedExp.value = this.awardedExperience;
	}
}
